@@ -1,6 +1,8 @@
 # SPDX-FileCopyrightText: 2022 James R. Barlow
 # SPDX-License-Identifier: MPL-2.0
 
+"""PDF metadata handling."""
+
 from __future__ import annotations
 
 import logging
@@ -28,6 +30,7 @@ else:
 if TYPE_CHECKING:  # pragma: no cover
     from pikepdf import Pdf
 
+
 XMP_NS_DC = "http://purl.org/dc/elements/1.1/"
 XMP_NS_PDF = "http://ns.adobe.com/pdf/1.3/"
 XMP_NS_PDFA_ID = "http://www.aiim.org/pdfa/ns/id/"
@@ -73,9 +76,11 @@ XPACKET_END = b"""\n<?xpacket end="w"?>\n"""
 
 
 class XmpContainer(NamedTuple):
+    """Map XMP container object to suitable Python container."""
+
     rdf_type: str
     py_type: type
-    insert_fn: Callable
+    insert_fn: Callable[..., None]
 
 
 log = logging.getLogger(__name__)
@@ -84,11 +89,9 @@ log = logging.getLogger(__name__)
 class NeverRaise(Exception):
     """An exception that is never raised."""
 
-    pass  # pylint: disable=unnecessary-pass
-
 
 class AltList(list):
-    pass
+    """XMP AltList container."""
 
 
 XMP_CONTAINERS = [
@@ -154,7 +157,7 @@ def _clean(s: str | Iterable[str], joiner: str = '; ') -> str:
 
 
 def encode_pdf_date(d: datetime) -> str:
-    """Encode Python datetime object as PDF date string
+    """Encode Python datetime object as PDF date string.
 
     From Adobe pdfmark manual:
     (D:YYYYMMDDHHmmSSOHH'mm')
@@ -173,7 +176,6 @@ def encode_pdf_date(d: datetime) -> str:
 
     'D:' is required in PDF/A, so we always add it.
     """
-
     # The formatting of %Y is not consistent as described in
     # https://bugs.python.org/issue13305 and underspecification in libc.
     # So explicitly format the year with leading zeros
@@ -187,7 +189,7 @@ def encode_pdf_date(d: datetime) -> str:
 
 
 def decode_pdf_date(s: str) -> datetime:
-    """Decode a pdfmark date to a Python datetime object
+    """Decode a pdfmark date to a Python datetime object.
 
     A pdfmark date is a string in a paritcular format. See the pdfmark
     Reference for the specification.
@@ -211,24 +213,34 @@ def decode_pdf_date(s: str) -> datetime:
 
 
 class Converter(ABC):
+    """XMP <-> DocumentInfo converter."""
+
     @staticmethod
     @abstractmethod
     def xmp_from_docinfo(docinfo_val: str | None) -> Any:  # type: ignore
-        "Derive XMP metadata from a DocumentInfo string"
+        """Derive XMP metadata from a DocumentInfo string."""
 
     @staticmethod
     @abstractmethod
     def docinfo_from_xmp(xmp_val: Any) -> str | None:
-        "Derive a DocumentInfo value from equivalent XMP metadata"
+        """Derive a DocumentInfo value from equivalent XMP metadata."""
 
 
 class AuthorConverter(Converter):
+    """Convert XMP document authors to DocumentInfo."""
+
     @staticmethod
     def xmp_from_docinfo(docinfo_val: str | None) -> Any:  # type: ignore
+        """Derive XMP authors info from DocumentInfo."""
         return [docinfo_val]
 
     @staticmethod
     def docinfo_from_xmp(xmp_val):
+        """Derive DocumentInfo authors from XMP.
+
+        XMP supports multiple author values, while DocumentInfo has a string,
+        so we return the values separated by semi-colons.
+        """
         if isinstance(xmp_val, str):
             return xmp_val
         if xmp_val is None or xmp_val == [None]:
@@ -237,14 +249,18 @@ class AuthorConverter(Converter):
 
 
 class DateConverter(Converter):
+    """Convert XMP dates to DocumentInfo."""
+
     @staticmethod
     def xmp_from_docinfo(docinfo_val):
+        """Derive XMP date from DocumentInfo."""
         if docinfo_val == '':
             return ''
         return decode_pdf_date(docinfo_val).isoformat()
 
     @staticmethod
     def docinfo_from_xmp(xmp_val):
+        """Derive DocumentInfo from XMP."""
         if xmp_val.endswith('Z'):
             xmp_val = xmp_val[:-1] + '+00:00'
         try:
@@ -256,6 +272,8 @@ class DateConverter(Converter):
 
 
 class DocinfoMapping(NamedTuple):
+    """Map DocumentInfo keys to their XMP equivalents, along with converter."""
+
     ns: str
     key: str
     name: Name
@@ -263,6 +281,11 @@ class DocinfoMapping(NamedTuple):
 
 
 def ensure_loaded(fn):
+    """Ensure the XMP has been loaded and parsed.
+
+    TODO: Can this be removed? Why allow the uninit'ed state to even exist?
+    """
+
     @wraps(fn)
     def wrapper(self, *args, **kwargs):
         if not self._xmp:
@@ -273,7 +296,7 @@ def ensure_loaded(fn):
 
 
 class PdfMetadata(MutableMapping):
-    """Read and edit the metadata associated with a PDF
+    """Read and edit the metadata associated with a PDF.
 
     The PDF specification contain two types of metadata, the newer XMP
     (Extensible Metadata Platform, XML-based) and older DocumentInformation
@@ -342,7 +365,7 @@ class PdfMetadata(MutableMapping):
     def load_from_docinfo(
         self, docinfo, delete_missing: bool = False, raise_failure: bool = False
     ) -> None:
-        """Populate the XMP metadata object with DocumentInfo
+        """Populate the XMP metadata object with DocumentInfo.
 
         Arguments:
             docinfo: a DocumentInfo, e.g pdf.docinfo
@@ -455,7 +478,7 @@ class PdfMetadata(MutableMapping):
             self._updating = False
 
     def _update_docinfo(self):
-        """Update the PDF's DocumentInfo dictionary to match XMP metadata
+        """Update the PDF's DocumentInfo dictionary to match XMP metadata.
 
         The standard mapping is described here:
             https://www.pdfa.org/pdfa-metadata-xmp-rdf-dublin-core/
@@ -508,7 +531,7 @@ class PdfMetadata(MutableMapping):
         return xml_bytes
 
     def _apply_changes(self):
-        """Serialize our changes back to the PDF in memory
+        """Serialize our changes back to the PDF in memory.
 
         Depending how we are initialized, leave our metadata mark and producer.
         """
@@ -533,7 +556,7 @@ class PdfMetadata(MutableMapping):
 
     @classmethod
     def _qname(cls, name: QName | str) -> str:
-        """Convert name to an XML QName
+        """Convert name to an XML QName.
 
         e.g. pdf:Producer -> {http://ns.adobe.com/pdf/1.3/}Producer
         """
@@ -557,7 +580,7 @@ class PdfMetadata(MutableMapping):
         return str(QName(uri, tag))
 
     def _prefix_from_uri(self, uriname):
-        """Given a fully qualified XML name, find a prefix
+        """Given a fully qualified XML name, find a prefix.
 
         e.g. {http://ns.adobe.com/pdf/1.3/}Producer -> pdf:Producer
         """
@@ -566,7 +589,7 @@ class PdfMetadata(MutableMapping):
         return self.REVERSE_NS[uri] + ':' + tag
 
     def _get_subelements(self, node):
-        """Gather the sub-elements attached to a node
+        """Gather the sub-elements attached to a node.
 
         Gather rdf:Bag and and rdf:Seq into set and list respectively. For
         alternate languages values, take the first language only for
@@ -598,7 +621,7 @@ class PdfMetadata(MutableMapping):
         return rdf
 
     def _get_elements(self, name: str | QName = ''):
-        """Get elements from XMP
+        """Get elements from XMP.
 
         Core routine to find elements matching name within the XMP and yield
         them.
@@ -792,7 +815,7 @@ class PdfMetadata(MutableMapping):
 
     @property
     def pdfa_status(self) -> str:
-        """Returns the PDF/A conformance level claimed by this PDF, or False
+        """Return the PDF/A conformance level claimed by this PDF, or False.
 
         A PDF may claim to PDF/A compliant without this being true. Use an
         independent verifier such as veraPDF to test if a PDF is truly
@@ -816,7 +839,7 @@ class PdfMetadata(MutableMapping):
 
     @property
     def pdfx_status(self) -> str:
-        """Returns the PDF/X conformance level claimed by this PDF, or False
+        """Return the PDF/X conformance level claimed by this PDF, or False.
 
         A PDF may claim to PDF/X compliant without this being true. Use an
         independent verifier such as veraPDF to test if a PDF is truly
