@@ -1,10 +1,5 @@
-/*
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
- *
- * Copyright (C) 2019, James R. Barlow (https://github.com/jbarlow83/)
- */
+// SPDX-FileCopyrightText: 2022 James R. Barlow
+// SPDX-License-Identifier: MPL-2.0
 
 #include <sstream>
 #include <type_traits>
@@ -19,6 +14,7 @@
 
 #include <qpdf/QPDFExc.hh>
 #include <qpdf/QPDFSystemError.hh>
+#include <qpdf/QPDFUsage.hh>
 #include <qpdf/QUtil.hh>
 #include <qpdf/Pl_Flate.hh>
 
@@ -52,10 +48,10 @@ enum pikepdf_error_type {
     error_type_cpp,
 };
 
-auto translate_qpdf_error(std::string msg)
+auto rewrite_qpdf_logic_error_msg(std::string msg)
 {
     using match_replace = std::pair<std::regex, std::string>;
-    pikepdf_error_type errtype;
+
     const static std::vector<match_replace> replacements = {
         match_replace{"QPDF::copyForeign(?:Object)?", "pikepdf.copy_foreign"},
         match_replace{"QPDFObjectHandle", "pikepdf.Object"},
@@ -65,6 +61,14 @@ auto translate_qpdf_error(std::string msg)
     for (auto mr : replacements) {
         msg = std::regex_replace(msg, mr.first, mr.second);
     }
+    return msg;
+}
+
+auto translate_qpdf_logic_error(std::string msg)
+{
+    pikepdf_error_type errtype;
+    msg = rewrite_qpdf_logic_error_msg(msg);
+
     if (std::regex_search(msg, std::regex("pikepdf.copy_foreign")))
         errtype = error_type_foreign;
     else if (std::regex_search(msg, std::regex("pikepdf.")))
@@ -74,9 +78,25 @@ auto translate_qpdf_error(std::string msg)
     return std::pair<std::string, pikepdf_error_type>(msg, errtype);
 }
 
-auto translate_qpdf_error(const std::exception &e)
+auto translate_qpdf_logic_error(const std::exception &e)
 {
-    return translate_qpdf_error(std::string(e.what()));
+    return translate_qpdf_logic_error(std::string(e.what()));
+}
+
+bool is_data_decoding_error(const std::runtime_error &e)
+{
+    static const std::regex decoding_error_pattern(
+        "character out of range"
+        "|broken end-of-data sequence in base 85 data"
+        "|unexpected z during base 85 decode"
+        "|TIFFPredictor created with"
+        "|Pl_LZWDecoder:"
+        "|Pl_Flate:"
+        "|Pl_DCT:"
+        "|stream inflate:",
+        std::regex_constants::icase);
+
+    return std::regex_search(e.what(), decoding_error_pattern);
 }
 
 PYBIND11_MODULE(_qpdf, m)
@@ -92,11 +112,13 @@ PYBIND11_MODULE(_qpdf, m)
     init_qpdf(m);
     init_pagelist(m);
     init_object(m);
+    init_job(m);
 
     // -- Support objects (alphabetize order) --
     init_annotation(m);
     init_embeddedfiles(m);
     init_nametree(m);
+    init_numbertree(m);
     init_page(m);
     init_parsers(m);
     init_rectangle(m);
@@ -117,8 +139,8 @@ PYBIND11_MODULE(_qpdf, m)
             "_test_file_not_found",
             []() -> void { (void)QUtil::safe_fopen("does_not_exist__42", "rb"); },
             "Used to test that C++ system error -> Python exception propagation works.")
-        .def("_translate_qpdf",
-            [](std::string s) { return translate_qpdf_error(s).first; })
+        .def("_translate_qpdf_logic_error",
+            [](std::string s) { return translate_qpdf_logic_error(s).first; })
         .def(
             "set_decimal_precision",
             [](uint prec) {
@@ -159,6 +181,7 @@ PYBIND11_MODULE(_qpdf, m)
     static py::exception<QPDFExc> exc_main(m, "PdfError");
     static py::exception<QPDFExc> exc_password(m, "PasswordError");
     static py::exception<QPDFExc> exc_datadecoding(m, "DataDecodingError");
+    static py::exception<QPDFUsage> exc_usage(m, "JobUsageError");
     static py::exception<std::logic_error> exc_foreign(m, "ForeignObjectError");
     py::register_exception_translator([](std::exception_ptr p) {
         try {
@@ -178,8 +201,10 @@ PYBIND11_MODULE(_qpdf, m)
             } else {
                 exc_main(e.what());
             }
+        } catch (const QPDFUsage &e) {
+            exc_usage(e.what());
         } catch (const std::logic_error &e) {
-            auto trans = translate_qpdf_error(e);
+            auto trans = translate_qpdf_logic_error(e);
             if (trans.second == error_type_foreign)
                 exc_foreign(trans.first.c_str());
             else if (trans.second == error_type_pdferror)
@@ -187,7 +212,7 @@ PYBIND11_MODULE(_qpdf, m)
             else
                 std::rethrow_exception(p);
         } catch (const std::runtime_error &e) {
-            if (str_startswith(e.what(), "character out of range"))
+            if (is_data_decoding_error(e))
                 exc_datadecoding(e.what());
             else
                 std::rethrow_exception(p);
